#!/usr/bin/perl -w
#	manageDevices - script to perform different operations for a list of devices
#	operations: add to discovery profile (new or existing), remove from profile and delete from system, change lifecycle state
#		add, addDiscover, remove, maintenance, retired, active
#	list of devices (by IP) provided in csv file
#	parameter file with CAPC host, tenant name, ipdomain name. Profile name can be specified or taken from iplist filename
#	discovery profile template file: DP_template.xml
#	by default, the script prints returncode information on stdout (verbose level=1)
#		it can be overridden by optional fourth parameter: 0=quiet, 1=item ids and some returncodes, 2=payloads, 3=filter select bodies
#	
# Prerequisites: tested with CA PM 3.5 and Perl 5.18
#	typically, REST::Client must be installed (cpan)
#
#	v1.1	June 2018
#
# DISCLAIMER
# This script is provided AS IS and no warranties are provided or made.  
# For more information refer to http://www.ca.com/us/legal/terms-for-user-sites.aspx

use strict;
use REST::Client;
##use Data::Dumper;		only used for debugging

my $num_args = $#ARGV + 1;
if ($num_args < 3 || $num_args > 4) {
  print "\nUsage: perl manageDevices.pl [add|addDiscover|remove|active|maintenance|retire] parameterFile ipFile [verbose]\n";
  print "Example: perl manageDevices.pl add param.ini 725.10.128.csv 1\n";
  exit;
}

my $CAPC_DA2PCurl = '/pc/center/webservice/datasources/dataSourceId/3/itemids';		## we assume the data source ID for the DA is 3
my $OP=$ARGV[0];
my $paramfile=$ARGV[1];
my $iplist=$ARGV[2];
my $verbose=1;			## default
if ($num_args > 3 ) {
	$verbose=$ARGV[3];
}
my %PARAMS;				# hash table with parameters
open(INPUT, $paramfile)
 or die "error when opening $paramfile for input $!\n";
# expected format: parameter=value\n
while (<INPUT>) {
	chomp;
	if ($_ =~ /(.+?)=(.*?)$/) {			# parse  NOTE: eol character
		$PARAMS{$1} = $2;				# add to hash
	}
}
close (INPUT);
#print Dumper(%PARAMS);
my $CAPC = $PARAMS{'CAPC'};
my $CADA = $PARAMS{'CADA'};
my $USER = $PARAMS{'USER'};				# global or tenant admin, only needed for CA PC based lifecycle state change
my $TENANT = $PARAMS{'TENANT'};
my $IPDOMAIN = $PARAMS{'IPDOMAIN'};
my $PROFILENAME = $PARAMS{'PROFILENAME'};

my @disctpl;										## CA DA discovery profile template
my $disctplsize = 0;
my $namepfx = "";
my $namesfx = "";
open(DISC, "DP_template.xml")						##	template with basic entries and variables, base for DP creation
  or die "Error by opening the template file $\n";	

while (<DISC>) {									## read the template file to get the profile name
    chomp;
	if ($_ =~ /<Name>(.*?)__NAME__(.*?)<\/Name>/) {		## check for any NAME placeholder prefix and suffix
		$namepfx = $1;
		$namesfx = $2;
	}
	push @disctpl, $_;
	$disctplsize++;
}
close(DISC);

if (not length($PROFILENAME)) {
	$iplist =~ /(.+)\.csv$/;
	$PROFILENAME = $namepfx . $1 . $namesfx;		## this is the full profile name including potential prefix/suffix in the template file
}
if ($verbose > 0) { print "profilename=$PROFILENAME \n"; }
my $tenantID = 0;
my $ipdomainID = 0;
my $profileID = 0;

my $capchost = "http://".$USER.'@'.$CAPC;
my $pcclient = REST::Client->new();
$pcclient->setHost($capchost);
$pcclient->addHeader('Content-Type', 'application/xml');
my $dahost = "http://".$CADA;						# tried to use CA PC as proxy to DA but it ended with rc=302
my $daclient = REST::Client->new();
$daclient->setHost($dahost);
$daclient->addHeader('Content-Type', 'application/xml');

###
## get tenant ID from DA
#################################################################################################
my $xmlbody = "";
$xmlbody = '<FilterSelect xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="filter.xsd">';
$xmlbody .= '<Filter>';
$xmlbody .= '<And>';
$xmlbody .= '<Tenant.Name type="EQUAL">' . $TENANT . '</Tenant.Name>';
$xmlbody .= '</And>';
$xmlbody .= '</Filter>';
$xmlbody .= '<Select use="exclude">';
$xmlbody .= '  </Select>';
$xmlbody .= '</FilterSelect>';
if ($verbose > 2) { print "$xmlbody \n"; }
$daclient->POST('/rest/tenants/filtered', $xmlbody);
if ($verbose > 1) { print "Tenant resolve request rc=", $daclient->responseCode(), "\n"; }
if ($verbose > 1) { print $daclient->responseContent(), "\n"; }
my $results = $daclient->responseContent();
if ($results =~ /.*\<ID\>(.+?)\<\/ID\>.*/) {			# parse out the tenant ids
	if ($verbose > 0) { print "tenantID=$1\n"; }
	$tenantID = $1;
}
###
## get ipdomain ID from DA
#################################################################################################
$xmlbody = '<FilterSelect xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="filter.xsd">';
$xmlbody .= '<Filter>';
$xmlbody .= '<And>';
$xmlbody .= '<IPDomain.Name type="EQUAL">' . $IPDOMAIN . '</IPDomain.Name>';
$xmlbody .= '</And>';
$xmlbody .= '</Filter>';
$xmlbody .= '<Select use="exclude">';
$xmlbody .= '  </Select>';
$xmlbody .= '</FilterSelect>';
if ($verbose > 2) { print "$xmlbody \n"; }
$daclient->POST('/rest/ipdomains/filtered', $xmlbody);
if ($verbose > 1) { print "IPdomain resolve request rc=", $daclient->responseCode(), "\n"; }
if ($verbose > 2) { print $daclient->responseContent(), "\n"; }
$results = $daclient->responseContent();
if ($results =~ /.*\<ID\>(.+?)\<\/ID\>.*/) {			# parse out the ipdomain ids
	if ($verbose > 0) { print "IPdomainID=$1\n"; }
	$ipdomainID = $1;
}
###
## get discover profile ID from DA (it may not exist yet)
#################################################################################################
$xmlbody = '<FilterSelect xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="filter.xsd">';
$xmlbody .= '<Filter>';
$xmlbody .= '<And>';
$xmlbody .= '<Item.Name type="EQUAL">' . $PROFILENAME . '</Item.Name>';
$xmlbody .= '</And>';
$xmlbody .= '</Filter>';
$xmlbody .= '<Select use="exclude">';
$xmlbody .= '  </Select>';
$xmlbody .= '</FilterSelect>';
if ($verbose > 2) { print "$xmlbody \n"; }
$daclient->POST('/rest/tenant/' . $tenantID . '/discoveryprofiles/filtered', $xmlbody);
if ($verbose > 1) { print "discovery profile resolve request rc=", $daclient->responseCode(), "\n"; }
if ($verbose > 2) { print $daclient->responseContent(), "\n"; }
$results = $daclient->responseContent();
if ($results =~ /.*\<ID\>(.+?)\<\/ID\>.*/) {			# parse out the ipdomain ids
	if ($verbose > 0) { print "profileID found=$1\n"; }
	$profileID = $1;
}

### read IP list of devices
##################################################################################################

my $devicelist;
my @devicetable;
my $deviceIP;
my $line = 0;

open(ASSOC, $iplist)
	or die "Error opening '$iplist': $!\n";
# expected format: IP, Name\n
while (<ASSOC>) {
	chomp;
	if ($_ =~ /(.+?),(.*?)$/ && $line > 0) {
		$deviceIP = $1;	
		push @devicetable, $deviceIP;
		if ($verbose > 2) { print $deviceIP, "\n"; }
	}
	$line++;
}
close (ASSOC);

my $itemname;

##
##  subroutines
##
sub createDiscProfile {
# create discovery profile from template with IP Domain and SNMP profile
#   params: 0: profile name, 1: ipdomain id,
		my $discproffile = "discProfile_" . $_[0] . ".xml";
		##open(DISC,"> " . $discproffile)					## optionally write profile also to file
		##	or die "Error opening '$discproffile': $!\n";
		$xmlbody = "";
		for (my $l = 0; $l < $disctplsize; $l++) {
			if ($disctpl[$l] =~ /<IPListList>/) {
				##print DISC "$disctpl[$l]\n";
				$xmlbody .= "$disctpl[$l]\n";
				for (my $m = 0; $m < scalar(@devicetable); $m++) {					## add IP addresses
					##print DISC "    <IPList>$devicetable[$m]</IPList>\n";
					$xmlbody .= "    <IPList>$devicetable[$m]</IPList>\n";
				}
			}
			elsif ($disctpl[$l] =~ /<Name>(.*?)<\/Name>/) {
				#$itemname = $disctpl[$l];					##	substitute placeholders for discovery profile name
				#$itemname =~ s/__NAME__/$_[0]/;			##  previous version
				$xmlbody .= "<Name>" . $_[0] . "</Name>\n";	##  now we have full profile name as parameter
			}
			elsif ($disctpl[$l] =~ /__IPDOMAINID__/) {
				$itemname = $disctpl[$l];					##	substitute placeholders for ip domain id
				$itemname =~ s/__IPDOMAINID__/$_[1]/;		
				##print DISC "$itemname\n";
				$xmlbody .= "$itemname\n";
			}
			else {
				##print DISC "$disctpl[$l]\n";
				$xmlbody .= "$disctpl[$l]\n";
			}
		}
		##close(DISC);
		if ($verbose > 2) { print $xmlbody; }
		$daclient->POST('/rest/tenant/' . $tenantID .'/discoveryprofiles/', $xmlbody);
		#$daclient->POST('/rest/tenant/1/discoveryprofiles/', $xmlbody);
		if ($verbose > 1) { print "profile create rc=" . $daclient->responseCode() . "\n"; }
		if ($verbose > 2) { print  . $daclient->responseContent() . "\n"; }
		$results = $daclient->responseContent();
		if ($results =~ /.*\<success\>(.+?)\<\/success\>.*/) {			# parse out the profileID
			if ($verbose > 0) { print "profileID created=$1\n"; }
			$profileID = $1;
		}
}
##
##
sub addToDiscProfile {
# add devices to discovery profile 
#   params: 0: profile name, 1: profileID
		my $discproffile = "discProfile_" . $_[0] . ".xml";
		##open(DISC,"> " . $discproffile)						## optionally write profile also to file
		##	or die "Error opening '$discproffile': $!\n";
		$xmlbody = '<DiscoveryProfile version="1.0.0"><AddList>';
		##print DISC "$xmlbody\n";
		for (my $m = 0; $m < scalar(@devicetable); $m++) {					## add host names or IP addresses
			##print DISC "    <Add>$devicetable[$m]</Add>\n";
			$xmlbody .= "    <Add>$devicetable[$m]</Add>\n";
		}
		$xmlbody .= '</AddList></DiscoveryProfile>';
		##print DISC "</AddList></DiscoveryProfile>\n";
		##close(DISC);
		if ($verbose > 2) { print $xmlbody; }
		if ($verbose > 1) { print '/rest/tenant/' . $tenantID .'/discoveryprofiles/' . $_[1] . "\n"; }
		$daclient->PUT('/rest/tenant/' . $tenantID .'/discoveryprofiles/' . $_[1], $xmlbody);
		if ($verbose > 0) { print "add to profile rc=" . $daclient->responseCode() . "\n"; }
		if ($verbose > 1) { print  $daclient->responseContent()."\n"; }
}
## 
##
sub runDiscProfile {
# run discovery profile 
#   params: 0: profile name, 1: profileID
		$xmlbody = '<DiscoveryProfile version="1.0.0">';
		$xmlbody .= '<RunStatus>START</RunStatus>';
		$xmlbody .= '</DiscoveryProfile>';
		if ($verbose > 2) { print $xmlbody; }
		if ($verbose > 1) { print '/rest/tenant/' . $tenantID .'/discoveryprofiles/' . $_[1] . "\n"; }
		$daclient->PUT('/rest/tenant/' . $tenantID .'/discoveryprofiles/' . $_[1], $xmlbody);
		if ($verbose > 0) { print "profile run rc=" . $daclient->responseCode() . "\n"; }
		if ($verbose > 1) { print  $daclient->responseContent()."\n"; }
}
##  
##
sub removeFromDiscProfile {
# remove devices from discovery profile 
#   params: 0: profile name, 1: profileID
		my $discproffile = "discProfile_" . $_[0] . ".xml";
		##open(DISC,"> " . $discproffile)					## optionally write profile also to file
		##	or die "Error opening '$discproffile': $!\n";
		$xmlbody = '<DiscoveryProfile version="1.0.0"><RemoveList>';
		##print DISC "$xmlbody\n";
		for (my $m = 0; $m < scalar(@devicetable); $m++) {					## add host names or IP addresses
			##print DISC "    <Remove>$devicetable[$m]</Remove>\n";
			$xmlbody .= "    <Remove>$devicetable[$m]</Remove>\n";
		}
		$xmlbody .= '</RemoveList></DiscoveryProfile>';
		##print DISC "</RemoveList></DiscoveryProfile>\n";
		##close(DISC);
		if ($verbose > 2) { print $xmlbody; }
		if ($verbose > 1) {  print '/rest/tenant/' . $tenantID .'/discoveryprofiles/' . $_[1] . "\n"; }
		$daclient->PUT('/rest/tenant/' . $tenantID .'/discoveryprofiles/' . $_[1], $xmlbody);
		if ($verbose > 0) { print "remove from profile rc=" . $daclient->responseCode() . "\n"; }
		if ($verbose > 1) { print $daclient->responseContent()."\n"; }
}
## 
sub deleteDevices {
## get device item IDs from DA
#   params: 0: ipdomain id,
#################################################################################################
	my $xmlbody = "";
	$xmlbody = '<FilterSelect xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="filter.xsd">';
	$xmlbody .= '<Filter>';
	$xmlbody .= '<And>';
	$xmlbody .= '<IPDomainMember.IPDomainID type="EQUAL">' . $_[0] . '</IPDomainMember.IPDomainID>';
	$xmlbody .= '<Or>';
	## <Device.PrimaryIPAddress type="EQUAL">130.119.103.8</Device.PrimaryIPAddress>
	for (my $m = 0; $m < scalar(@devicetable); $m++) {					## add host names or IP addresses
		$xmlbody .= '<Device.PrimaryIPAddress type="EQUAL">' . $devicetable[$m] . '</Device.PrimaryIPAddress>';
	}
	$xmlbody .= '</Or>';
	$xmlbody .= '</And>';
	$xmlbody .= '</Filter>';
	$xmlbody .= '<Select use="exclude">';
	$xmlbody .= '  </Select>';
	$xmlbody .= '</FilterSelect>';
	if ($verbose > 2) { print "$xmlbody \n"; }
	$daclient->POST('/rest/devices/filtered', $xmlbody);
	if ($verbose > 0) { print "devices remove request rc=", $daclient->responseCode(), "\n"; }
	if ($verbose > 1) { print $daclient->responseContent(), "\n"; }
	my $results = $daclient->responseContent();
	my @rows;
	(@rows) = ($results =~ m@(<ID>.+?</ID>)@g); 			# make each result record a row
	##print Dumper(@rows);
	$xmlbody = "<DeleteList>";
	#print "deleted items:\n";
	foreach my $r (@rows) { 
		#print $r, "\n";
		$xmlbody .= $r;
	}
	$xmlbody .= "</DeleteList>";
	  if ($verbose > 2) { print $xmlbody, "\n"; }
	  $daclient->POST('/rest/devices/deletelist', $xmlbody);	
	  if ($verbose > 0) { print  "delete devices rc=" . $daclient->responseCode(), "\n"; }

}
##
## 
sub changeDevicesLC {
## get device item IDs from DA
#   params: 0: $OP, 1: ipdomain id,
#################################################################################################
	my $xmlbody = "";
	$xmlbody = '<FilterSelect xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="filter.xsd">';
	$xmlbody .= '<Filter>';
	$xmlbody .= '<And>';
	$xmlbody .= '<IPDomainMember.IPDomainID type="EQUAL">' . $_[1] . '</IPDomainMember.IPDomainID>';
	$xmlbody .= '<Or>';
	## <Device.PrimaryIPAddress type="EQUAL">130.119.103.8</Device.PrimaryIPAddress>
	for (my $m = 0; $m < scalar(@devicetable); $m++) {					## add host names or IP addresses
		$xmlbody .= '<Device.PrimaryIPAddress type="EQUAL">' . $devicetable[$m] . '</Device.PrimaryIPAddress>';
	}
	$xmlbody .= '</Or>';
	$xmlbody .= '</And>';
	$xmlbody .= '</Filter>';
	$xmlbody .= '<Select use="exclude">';
	$xmlbody .= '  </Select>';
	$xmlbody .= '</FilterSelect>';
	if ($verbose > 2) { print "$xmlbody \n"; }
	$daclient->POST('/rest/devices/filtered', $xmlbody);
	if ($verbose > 1) { print "get devices request rc=", $daclient->responseCode(), "\n"; }
	if ($verbose > 2) { print $daclient->responseContent(), "\n"; }
	my $results = $daclient->responseContent();
	my @rows;
	(@rows) = ($results =~ m@(<ID>.+?</ID>)@g); 			# make each result record a row
	## translate these DA device item IDs to PC item IDs
	$xmlbody = "<LocalIDs>";
	foreach my $r (@rows) { 
		# print $r, "\n";
		if ($r =~ /\<ID\>(.+?)\<\/ID\>/) {			# parse out the item ids
			#print "itemidDA=$1\n";
			$xmlbody .= '<LocalID ID="' . $1 . '"/>';
		}
	}
	$xmlbody .= "</LocalIDs>";
	if ($verbose > 2) { print $xmlbody, "\n"; }
	$pcclient->POST($CAPC_DA2PCurl, $xmlbody);	
	if ($verbose > 1) { print "retrieve PC item IDs rc=", $pcclient->responseCode(), "\n"; }
	if ($verbose > 2) { print  $pcclient->responseContent(), "\n"; }
	$results = $pcclient->responseContent();

	$xmlbody .= "<emptybody/>";
	my $op = uc $_[0];
	(@rows) = ($results =~ m@(<ItemIDResult.+?\/>)@ig); 			# make each result line a row
	foreach my $r (@rows) { 
		# print $r, "\n";
		if ($r =~ /LocalID=\"(.+?)\" ItemID=\"(.+?)\"/) {			# parse out the item ids
			if ($verbose > 1) { print "itemIDPC=$2,itemidDA=$1 \n"; }
			# $DAitem{$2} = $1;										# add to hash
			if ($verbose > 1) { print '/pc/center/webservice/devices/deviceItemId/' . $2 . '/lifeCycleState/' . $op . "\n"; }
			$pcclient->PUT('/pc/center/webservice/devices/deviceItemId/' . $2 . '/lifeCycleState/' . $op, $xmlbody);	
			if ($verbose > 0) { print "change state for PC item ", $2 , " rc=" , $pcclient->responseCode(), "\n"; }
			if ($verbose > 2) { print  $pcclient->responseContent(), "\n"; }
			sleep 1;
		}
	}

}
##
##############################################################################################
if ($OP eq "add" || $OP eq "addDiscover" ) {
	if ($profileID == 0) {
		createDiscProfile($PROFILENAME, $ipdomainID);
		sleep 1;
	} else {
		addToDiscProfile($PROFILENAME, $profileID);
	}
	if ($OP eq "addDiscover" ) {
		runDiscProfile($PROFILENAME, $profileID);
	}
}
if ($OP eq "remove" && $profileID > 0 ) {
	removeFromDiscProfile($PROFILENAME, $profileID);
	deleteDevices($ipdomainID);
}
if ($OP eq "maintenance" || $OP eq "retired" || $OP eq "active" ) {
	changeDevicesLC($OP, $ipdomainID);
}
